#ifndef _USB_INTERFACE_H
#define _USB_INTERFACE_H


#include <stdint.h>

#include <exception>
#include <string>
#include <vector>

#include <string.h>



class UsbException : public std::exception
{
public:
	UsbException(const char *message) noexcept
	{
		mMessage = new char[strlen(message) + 1];
		strcpy(mMessage, message);
	}

	~UsbException() noexcept
	{
		delete[] mMessage;
	}

	const char* what() const noexcept			///<Returns the error message
	{
		return mMessage;
	}

private:
	char *mMessage;
};



class UsbDeviceInfo
{
public:
	uint16_t vid;
	uint16_t pid;

	uint16_t bcdDevice;
	
	std::string vendor;
	std::string product;
	std::string serial;

private:
#if defined (__GNUC__)
	void *handle;
#elif defined (_MSC_VER)	
	std::string device_path;
#endif

	friend class UsbDriver;
};


typedef std::vector<UsbDeviceInfo> UsbDeviceInfoList;



class UsbInterface
{
public:
	virtual void open(UsbDeviceInfo &device_info) = 0;
	virtual bool isOpened(void) const = 0;
	virtual void close(void) = 0;
	
	virtual void find(UsbDeviceInfoList &list) = 0;
};



#endif

